@@ -883,11 +883,11 @@ class Client(object):
         qos: Not used.
 
         The function returns a tuple (result, mid), where result is
-        MQTT_ERR_SUCCESS to indicate success or MQTT_ERR_NO_CONN if the client
-        is not currently connected.  mid is the message ID for the subscribe
-        request. The mid value can be used to track the subscribe request by
-        checking against the mid argument in the on_subscribe() callback if it
-        is defined.
+        MQTT_ERR_SUCCESS to indicate success or (MQTT_ERR_NO_CONN, None) if the
+        client is not currently connected.  mid is the message ID for the
+        subscribe request. The mid value can be used to track the subscribe
+        request by checking against the mid argument in the on_subscribe()
+        callback if it is defined.
 
         Raises a ValueError if qos is not 0, 1 or 2, or if topic is None or has
         zero string length, or if topic is not a string, tuple or list.
@@ -917,7 +917,7 @@ class Client(object):
             raise ValueError("No topic specified, or incorrect topic type.")
 
         if self._sock is None and self._ssl is None:
-            return MQTT_ERR_NO_CONN
+            return (MQTT_ERR_NO_CONN, None)
 
         return self._send_subscribe(False, topic_qos_list)
 
@@ -928,8 +928,8 @@ class Client(object):
                topics to unsubscribe from.
 
         Returns a tuple (result, mid), where result is MQTT_ERR_SUCCESS
-        to indicate success or MQTT_ERR_NO_CONN if the client is not currently
-        connected.
+        to indicate success or (MQTT_ERR_NO_CONN, None) if the client is not
+        currently connected.
         mid is the message ID for the unsubscribe request. The mid value can be
         used to track the unsubscribe request by checking against the mid
         argument in the on_unsubscribe() callback if it is defined.
@@ -954,7 +954,7 @@ class Client(object):
             raise ValueError("No topic specified, or incorrect topic type.")
 
         if self._sock is None and self._ssl is None:
-            return MQTT_ERR_NO_CONN
+            return (MQTT_ERR_NO_CONN, None)
 
         return self._send_unsubscribe(False, topic_list)
 
@@ -1254,7 +1254,8 @@ class Client(object):
                     return MQTT_ERR_AGAIN
                 if msg.errno == EAGAIN:
                     return MQTT_ERR_AGAIN
-                raise
+                print(msg)
+                return 1
             else:
                 if len(command) == 0:
                     return 1
@@ -1277,7 +1278,8 @@ class Client(object):
                         return MQTT_ERR_AGAIN
                     if msg.errno == EAGAIN:
                         return MQTT_ERR_AGAIN
-                    raise
+                    print(msg)
+                    return 1
                 else:
                     byte = struct.unpack("!B", byte)
                     byte = byte[0]
@@ -1308,7 +1310,8 @@ class Client(object):
                     return MQTT_ERR_AGAIN
                 if msg.errno == EAGAIN:
                     return MQTT_ERR_AGAIN
-                raise
+                print(msg)
+                return 1
             else:
                 self._in_packet['to_process'] = self._in_packet['to_process'] - len(data)
                 self._in_packet['packet'] = self._in_packet['packet'] + data
@@ -1354,7 +1357,8 @@ class Client(object):
                     return MQTT_ERR_AGAIN
                 if msg.errno == EAGAIN:
                     return MQTT_ERR_AGAIN
-                raise
+                print(msg)
+                return 1
 
             if write_length > 0:
                 packet['to_process'] = packet['to_process'] - write_length
@@ -1383,6 +1387,13 @@ class Client(object):
                             self.on_disconnect(self, self._userdata, 0)
                             self._in_callback = False
                         self._callback_mutex.release()
+
+                        if self._ssl:
+                            self._ssl.close()
+                            self._ssl = None
+                        elif self._sock:
+                            self._sock.close()
+                            self._sock = None
                         return MQTT_ERR_SUCCESS
 
                     self._out_packet_mutex.acquire()
